
/****************************************************************************

  William E. Ives
  Nov 16, 1988

  This program is an attempt to use the command interpreter
  as a child process connected to this parent process via pipes.

*****************************************************************************/

#include <mt\stdio.h>
#include <mt\stddef.h>
#include <mt\stdlib.h>
#include <mt\process.h>
#include <mt\ctype.h>
#include <mt\conio.h>
#include <mt\string.h>
#include <mt\malloc.h>

#define INCL_BASE
#define INCL_DOSPROCESS
#include "mt\os2.h"

#define PIPESIZE 100

#define STDIN 0
#define STDOUT 1
#define TRUE 1
#define FALSE 0

/****************************************************************************
  connect

  This routine starts the specified child process 'childname' as
  an asyncronous process with its stdin/stdout connected to
  two pipes.  The handles to these pipes are passed back to the
  caller in 'rhandle' and 'whandle' which correspond to
  the childs stdout and stdin handles respectively.

  Input :
     childname : name of child process
  Output:
     childpid : process ID of child.
     rhandle  : pipe read handle that can be used to receive
		information from the child's stdout.
     whandle  : pipe write handle that can be used to send
		information to the child's stdin.
  Return :
    0 : connection established okay.
    other : error.
*****************************************************************************/
int  connect ( char far * childname, PID far * childpid ,
	       HFILE far * rhandle , HFILE far * whandle )
{
  char	   achFailName[128];
  RESULTCODES rescResults;
  USHORT   retcode , fstate ;
  HFILE    r1handle, w1handle , mystdout, childout, mystdin, childin ;

  if ( DosMakePipe( rhandle, &w1handle, PIPESIZE+1 ) ){
     return 1 ;
     }
  if ( DosMakePipe(&r1handle, whandle, PIPESIZE+1 ) ){
     DosClose( *rhandle );
     DosClose( w1handle );
     return 1 ;
     }
  DosQFHandState( *rhandle,(PUSHORT) &fstate );
  DosSetFHandState( *rhandle,
	     (USHORT)fstate | 0x0080); /* mark read handle as uninheritable */
  DosQFHandState( *whandle,(PUSHORT) &fstate );
  DosSetFHandState( *whandle,
	     (USHORT)fstate | 0x0080); /* mark write handle as uninheritable */

  /* substitute standard handle for child */
  mystdout=-1;
  DosDupHandle(STDOUT,&mystdout);
  DosClose(STDOUT);
  childout = STDOUT ;
  DosDupHandle( w1handle, &childout ); /* write handle of pipe 1 ==STDOUT */
  mystdin=-1;
  DosDupHandle(STDIN,&mystdin);
  DosClose(STDIN);
  childin = STDIN ;
  DosDupHandle( r1handle, &childin ); /* read handle of pipe 2 ==STDIN */

  DosQFHandState( w1handle, &fstate );
  DosSetFHandState( w1handle,
	     (USHORT)fstate | 0x0080); /* mark write handle as uninheritable */
  DosQFHandState( r1handle, &fstate );
  DosSetFHandState( r1handle,
	     (USHORT)fstate | 0x0080); /* mark read handle as uninheritable */

  retcode =  DosExecPgm( achFailName, sizeof(achFailName),EXEC_ASYNC,
			 NULL,NULL,&rescResults,childname);

  /* restore my stdout */
  childout = STDOUT ;
  DosDupHandle(mystdout,&childout);
  DosClose(w1handle);
  DosClose(mystdout);

  childin  = STDIN  ;
  DosDupHandle(mystdin ,&childin );
  DosClose(r1handle);
  DosClose(mystdin);

  if ( retcode ) {
     puts(achFailName);
     return 1 ;
     }

  *childpid = rescResults.codeTerminate ;
  return 0;
}

char * comspec = "c:\\os2\\pbin\\cmd.exe";
PID    comPID ;
HFILE  comin, comout ;
char   str[PIPESIZE],buf[PIPESIZE];
char   com_installed = FALSE ;

/****************************************************************************
  ExitListHandler

  This routine is a user installed exit list handler which ensures
  that the child process is killed before termination. It also
  closes the pipes handles.

*****************************************************************************/
void far pascal ExitListHandler()
{
  if ( com_installed ) {
    DosKillProcess(DKP_PROCESSTREE, comPID );
    DosClose(comin);
    DosClose(comout);
    }
  DosExitList(EXLST_EXIT, (void far *) ExitListHandler);
}

main()
{

  USHORT bytesread, byteswritten ;

  if ( DosExitList(EXLST_ADD, (void far *)ExitListHandler)) {
	printf(" Unable to install exit list handler.\n");
	return ;  /* this still does a DosExit */
	}

  if ( connect( comspec, &comPID, &comin, &comout) ) {
     printf(" Error starting command interpreter\n");
     return;
     }
  com_installed = TRUE ;


  for (;;) {
      DosRead( STDIN,(PVOID)str, PIPESIZE,(PUSHORT)&bytesread);
      if ( DosWrite( comout, (PVOID) str, bytesread,
		     (PUSHORT) &byteswritten ) ) {
	  printf(" Write error to command interpreter\n");
	  return;
	  }
      do {
	 if ( DosRead( comin,(PVOID) buf, PIPESIZE,(PUSHORT)&bytesread)) {
	    printf(" Read error from command interpreter\n");
	    return;
	    }
	 DosWrite( STDOUT,(PVOID)buf,bytesread,(PUSHORT)&byteswritten);
	 } while ( bytesread );
      break;
      }
}

