
/*
   William E. Ives
   Nov 1, 1988

   A keyboard monitor that implements an entry retrieval
   for OS/2 command line.

*/

#define INCL_SUB
#define INCL_DOS

#define TRUE 1
#define FALSE 0

#include <os2.h>
#include <mt/process.h>
#include <mt/stdio.h>
#include <mt/stdlib.h>

/* The format of keyboard monitor packets is as follows: */
/*
	NOTES:
		origScan should be set to zero for packets inserted by the
		keyboard monitor process.
*/
typedef struct {
	UCHAR monFlag;		/* Keyboard monitor packet flag */
	UCHAR origScan; 	/* Original hardware scan code */
	KBDKEYINFO key; 	/* Same as info for KbdCharIn */
	USHORT kbdDDFlags;	/* */
	} kbdpacket_type ;

typedef struct	{
     unsigned size ;
     char     reserved[18];
     char     buf[108];
     } buffer_type;

typedef enum { backspace = 8 , enter = 13, escape = 27, cntrl_f1 = 94,
	       up_arrow = 72, down_arrow = 80 } key_codes ;



/* Values for monFlag field */

#define MF_OPEN 	0
#define MF_CLOSE	1
#define MF_FLUSH	2

/* Values for kbdDDFlags */
#define KDD_BREAK	0x040	/* The packet is a key-release event */
#define KDD_SECONDARY	0x080	/* Prior scan code is Secondary Key Prefix */
#define KDD_MULTIMAKE	0x100	/* Auto-repeat of a toggle key */
#define KDD_ACCENTED	0x200	/* Previous key was an accent key */

/* Values for key.fsState field */

#define KS_SHIFT	0x0003
#define KS_RSHIFT	0x0001
#define KS_LSHIFT	0x0002
#define KS_CTRL 	0x0004
#define KS_ALT		0x0008
#define KS_NOSCROLL	0x0010
#define KS_NUMONLY	0x0020
#define KS_CAPSONLY	0x0040
#define KS_INSERT	0x0080
#define KS_LCTRL	0x0100
#define KS_LALT 	0x0200
#define KS_RCTRL	0x0400
#define KS_RALT 	0x0800
#define KS_SCRLLOCK	0x1000
#define KS_NUMLOCK	0x2000
#define KS_CAPSLOCK	0x4000
#define KS_SYSREQ	0x8000


ULONG term_sem = 0L ;
unsigned tid ;

/* thread stack */
char stack1[4096] ;

/* command stack */
#define MAXCMDS 10
char  cmd[MAXCMDS][80];


void far keymon()
{
  USHORT	 sessionID , DOS_PID ;
  HMONITOR	 monitor_handle ;
  buffer_type	 inbuf,outbuf;
  kbdpacket_type mybuf ;

  USHORT   len ;
  USHORT   wait ;
  SEL	   gsel, lsel ;
  GINFOSEG far * info ;
  char	   * str ;
  int	   next_command;
  char	   cur_cmd[100];
  int	   c ,i, j ;

  j = 0;
  next_command=0;
  c =0;
  cur_cmd[0]=0;

  /* open the monitor  */
  if ( DosMonOpen( "KBD$",(PHMONITOR)&monitor_handle ) ) {
     printf(" Cannot open keyboard monitor.\n");
     DosBeep(600,175);
     DosSemClear( (HSEM) &term_sem);
     return ;
     }

  inbuf.size = sizeof( buffer_type );
  outbuf.size = sizeof( buffer_type );

  /* get the current screen group ID */
  DosGetInfoSeg( &gsel, &lsel ) ;
  info = MAKEP(gsel,0);
  sessionID = info->sgCurrent ;
  DOS_PID = info->pidForeground ;

  /* up this thread's priority */
  if ( DosSetPrty( PRTYS_THREAD,     /* change only this thread */
		   PRTYC_TIMECRITICAL,
		   16,		     /* leave at lowest priority */
		   tid)) {
     printf(" Unable to make the keymon() thread into time-critical.\n");
     DosBeep(600,175);
     }

   /* register the monitor at the start of the chain */
   if (DosMonReg( monitor_handle, (PBYTE)&inbuf,(PBYTE)&outbuf,1,sessionID)){
      printf(" Cannot register keyboard monitor.\n");
      DosBeep(600,175);
      DosMonClose(monitor_handle);
      DosSemClear( (HSEM) &term_sem);
      return ;
      }

   /* this is the main monitor loop */
   for (;;) {
      len = sizeof( buffer_type );
      mybuf.key.chScan = 0 ;  /* clear scan code each time. */

      /* read the monitor buffer */
      DosMonRead( (PBYTE)&inbuf,DCWW_WAIT,(PBYTE)&mybuf,&len);

      /* only process input from the OS/2 command interpreter */
      if ( info->pidForeground == DOS_PID ) {

	 /* wait for a break code if it is the up-arrow or down-arrow */
	 if ( (mybuf.kbdDDFlags & KDD_BREAK)&&
	      ( (mybuf.key.chScan==up_arrow)||(mybuf.key.chScan==down_arrow) ) ) continue ;

	 /* if arrow is pressed insert last command into monitor buffer */
	 if ( (mybuf.key.chScan == up_arrow)||(mybuf.key.chScan==down_arrow) ) {

	    if (c) {	  /* send out back spaces to erase current line */
	       for (;c;c--){
		mybuf.kbdDDFlags &= ~KDD_BREAK; /* clear break */
		mybuf.key.chChar = backspace ;
		DosMonWrite((PBYTE)&outbuf,(PBYTE)&mybuf,len);
		mybuf.kbdDDFlags |= KDD_BREAK;	/* set break */
		DosMonWrite((PBYTE)&outbuf,(PBYTE)&mybuf,len);
		}
	       }

	    if ( mybuf.key.chScan==up_arrow)  /* if up-arrow then get prev command */
	      if ( next_command ) next_command -- ;
	      else next_command = MAXCMDS-1;
	    else		   /* if down-arrow then inc next_command */
	      next_command =(next_command==MAXCMDS-1)? 0:next_command+1;

	    str = cmd[next_command];
	    c = strlen(str);
	    strcpy(cur_cmd,str);

	    for (;*str;str++) {
		mybuf.kbdDDFlags &= ~KDD_BREAK; /* clear break */
		mybuf.key.chChar = *str ;
		DosMonWrite((PBYTE)&outbuf,(PBYTE)&mybuf,len);
		mybuf.kbdDDFlags |= KDD_BREAK;	/* set break */
		DosMonWrite((PBYTE)&outbuf,(PBYTE)&mybuf,len);
		}
	    }
	 else {
	    DosMonWrite( (PBYTE) &outbuf, (PBYTE) &mybuf,len);

	    /* examine key only after break code */
	    if ( !(mybuf.kbdDDFlags & KDD_BREAK)) continue ;

	    if ( mybuf.key.chScan == cntrl_f1) break ;	/* exit monitor if c-F1 is pressed */

	    if ( (mybuf.key.chChar != 0) && (mybuf.key.chChar != 0xE0)) {
	       switch ( mybuf.key.chChar ) {
		  case enter:/* enter pressed , save the command. */
			    strcpy(cmd[next_command],cur_cmd);
			    next_command++;
			    if( next_command==MAXCMDS) next_command=0;
			    c=0;
			    cur_cmd[0]=0;
			    break;
		  case backspace  : /* backspace */
			    if (c) {
			       c--;
			       strcpy(cur_cmd+c,cur_cmd+c+1);
			       }
			    break;
		  case escape : break;
		  default :
			    if (mybuf.key.fsState & KS_INSERT) { /* if insert on */
			       for (i=strlen(cur_cmd);
				    i>=c;i--) cur_cmd[i+1]=cur_cmd[i];
			       cur_cmd[c]=mybuf.key.chChar;
			       c++;
			       }
			    else {
			      cur_cmd[c]=mybuf.key.chChar ;
			      c++;
			      cur_cmd[c]=0;
			      }
		  }
	       }
	    }
	 }
      else {
	 DosMonWrite( (PBYTE)&outbuf, (PBYTE)&mybuf, len );

	 /* examine key only after break code */
	 if ( !(mybuf.kbdDDFlags & KDD_BREAK)) continue ;

	 if ( mybuf.key.chScan == cntrl_f1) break ;  /* exit monitor if c-F1 is pressed */
	 }

      /* if OS/2 command pid goes away force a deinstall */
      /* but only check this every 10 keys so that it does not
	  slow down the system */
      j++;
      if ( j==10 ) {
	if ( DosGetPrty(PRTYS_PROCESS, &wait, DOS_PID ) ) break ;
	j = 0;
	}

      } /* end for loop */

   DosMonClose(monitor_handle);
   DosSemClear( &term_sem);

} /* keymon */


main()
{
  USHORT wait ;

  for ( wait=0;wait<MAXCMDS;wait++) cmd[wait][0]=0;

  DosSemSet( ( ULONG far * ) &term_sem );

  tid = _beginthread(keymon, (void far *) stack1, 4096, (void far *)0);

  /* wait forever on term_sem */
  DosSemWait( (unsigned far*) &term_sem,-1L);

}



