
/****************************************************************************

  OS/2 Command Line Editor/Retriever Dynamic Link Library

  NOTE:
     The MTDYNA.DOC file states that a DLL must be compiled with
     -Alfw. This is not true -- you really need Axxu, where xx is of
     your choice. For a small model DLL you could use -Asnu.

  Author: William E. Ives
  Date	: November 18, 1988  first working version

****************************************************************************/

#include <mt\stdio.h>
#include <mt\stddef.h>
#include <mt\stdlib.h>
#include <mt\string.h>
#include <mt\conio.h>
#include <mt\ctype.h>

#define INCL_BASE
#define INCL_SUB
#include <mt\os2.h>

int _acrtused = 0;  /*	 do not include the runtime library. */

#include "keys.h"

BYTE tattr = 7 , norm_attr=7, high_attr=7, x_col , y_row;
#define gotoxy( x, y )	VioSetCurPos(y,x,0); x_col = x; y_row = y ;
#define textattr( att ) tattr = att ;
#define cputs( st )	VioWrtCharStrAtt( st ,strlen( st ),y_row,x_col,&tattr,0);
#define cputch( ch )	VioWrtCharStrAtt(&ch,1,y_row,x_col,&tattr,0);


VIOINTENSITY  viostat ;
VIOCURSORINFO viocurstat ;
VIOFONTINFO   viofinfo ;

/* function prototypes */
void SetCursorShape ( int shape );


/****************************************************************************
 InitVioKbd

     This routine initializes internal variable for the DLL and is callable
     from outside the DLL.  It also sets the video state so that
     the background intensity bit is enabled and gets the font size for
     cursor resizing.

 Input :
    nattr : the normal key entry screen attribute
    hattr : the highlighted key entry screen attribute

****************************************************************************/
void far pascal InitVioKbd( int nattr, int hattr )
{
  static VIOINTENSITY tviostat ;

  viostat.cb = sizeof(VIOINTENSITY);
  viostat.type = 2 ;
  if ( VioGetState(&viostat,0) ) return ;	     /* save vio state */
  memcpy(&tviostat,&viostat,sizeof(VIOINTENSITY) );

  if ( VioGetCurType(&viocurstat,0) ) return ;	     /* save cursor state */

  viofinfo.cb=sizeof(VIOFONTINFO);
  viofinfo.type = VGFI_GETCURFONT ;
  viofinfo.pbData = 0L;
  VioGetFont(&viofinfo,0);  /* get font info for use with cursor resizing */

  tviostat.type =2;	   /* allows the setting of background intensity bit */
  tviostat.fs =1;	   /* sets high intensity background */
  VioSetState(&tviostat,0);

  norm_attr=nattr;
  high_attr=hattr;

} /* InitVioKbd */


#define MAXSAVE 9
#define MAXCMDLEN 100
char save[MAXSAVE+1][MAXCMDLEN]={ "","","","","","","","","","" } ;
int  cursave=0;


/****************************************************************************
 INPUTSTRING

     This routine replaces the KbdStringIn DLL routine.  It interacts with
     the user and allows the editing and retrieving of previous command
     lines.

****************************************************************************/
int  far INPUTSTRING ( SEL DataSeg ,
		       USHORT res1 ,
		       USHORT fFunction,
		       ULONG  res2 ,
		       USHORT param1,
		       USHORT param2,
		       USHORT param3,
		       USHORT param4,
		       USHORT param5,
		       USHORT param6)
{

  static int	  len,i, max , count ;
  static int	  insert_mode ,c,r, start_c ;
  static unsigned char	chChar ;
  static PCH	  str ;
  static PSTRINGINBUF pLength ;
  static USHORT   fNoWait   ;

  extern void save_regs();
  extern void restore_regs();

  save_regs();

  /* get parameters off the stack from the keystringin call */
  str = (PCH) MAKEP( param6, param5 );
  pLength = (PSTRINGINBUF) MAKEP( param4, param3 );
  fNoWait = param2 ;


  max= pLength->cb ;
  insert_mode = FALSE;
  SetCursorShape ( insert_mode );
  len = 0 ;
  c=0;
  str[0]=0;
  VioGetCurPos((PUSHORT)&r,(PUSHORT)&start_c,0);
  gotoxy(start_c,r);

  textattr(norm_attr);
  for(;;) {

     if ( fNoWait==IO_NOWAIT )
	if ( kbhit() )
	    chChar = getch();
	else {
	   SetCursorShape(0);
	   pLength->cchIn = c ;
	   restore_regs();
	   return 0;
	   }
     else
	chChar = getch();


     if ( ( chChar == 0 ) || ( chChar ==  0xE0 ) )
       switch ( chChar=getch()) {
	  case Up : /* up = retrieve last command */
		    for (c=0;c<len;c++) str[c]=' ';
		    gotoxy(start_c,r);
		    textattr(high_attr);
		    cputs(str);
		    textattr(norm_attr);
		    cursave=(cursave)? cursave-1:MAXSAVE;
		    strncpy(str,(char far *)save[cursave],max);
		    str[max-1]=0;
		    len=strlen(str);
		    c=len;
		    cputs(str);
		    break;
	  case Down: /* down = retrieve next command */
		    cursave=(cursave==MAXSAVE)? 0: cursave+1;
		    for (c=0;c<len;c++) str[c]=' ';
		    gotoxy(start_c,r);
		    textattr(high_attr);
		    cputs(str);
		    textattr(norm_attr);
		    strncpy(str,(char far *)save[cursave],max);
		    str[max-1]=0;
		    len=strlen(str);
		    c=len;
		    cputs(str);
		    break;
	  case Right : c=(c<len)?c+1:c ; break ; /* right */
	  case Left  : c=(c)?c-1:c ;  break ; /* left */
	  case Home  : c=0 ; break ; /* home */
	  case End   : c=len; break; /* end */
	  case Insert: insert_mode = ! insert_mode ;  /* insert */
		       SetCursorShape ( ( insert_mode )?1:0 );
		       break;
	  case Delete: if ( str[c] ) {	 /* delete current char */
			   strcpy(str+c,str+c+1);
			   gotoxy(start_c+len-1,r);
			   chChar=' ';
			   textattr(high_attr);
			   cputch(chChar);
			   textattr(norm_attr);
			   gotoxy(start_c+c,r);
			   cputs(str+c);
			   len--;
			   }
			 break;
	  default : /* ignor undefined extended keys */
		   break;
	  }
     else {
	switch ( chChar ) {
	   case Enter :
		    /* save the string in a retrieve buffer */
		    strcpy((char far *) save[cursave], str);
		    cursave=(cursave==MAXSAVE)? 0: cursave+1;

		    gotoxy(start_c,r);	 /* highlight the string. */
		    textattr(high_attr);
		    cputs(str);
		    gotoxy(0,r);

		    c=len;
		    str[c]=chChar ;
		    str[len+1]=0;

		    SetCursorShape (0); /* restore line cursor */
		    pLength->cchIn = len ;

		    restore_regs();
		    return 0 ;

	   case Escape : /* erase line if ESC is pressed. */
		     for (c=0;c<len;c++) str[c]=' ';
		     gotoxy(start_c,r);
		     textattr(high_attr);
		     cputs(str);
		     textattr(norm_attr);
		     str[0]=0;
		     c=0;
		     len=0;
		     break;

	   case Backspace : /* handle detructive back space */
		     if (c) {
			c--;
			strcpy(str+c,str+c+1);
			gotoxy(start_c+len-1,r);
			chChar=' ';
			textattr(high_attr);
			cputch(chChar);
			textattr(norm_attr);
			gotoxy(start_c+c,r);
			cputs(str+c);
			len--;
			}
		     break;
	   default :
		     if ( ! isprint(chChar) ) {
		       }
		     else
		       if ( insert_mode ) {
			  for (i=len;i>=c;i--) str[i+1]=str[i];
			  str[c]=chChar ;
			  len++;
			  cputs(str+c);
			  c++;
			  if (c==max) {
			     SetCursorShape(0);
			     pLength->cchIn = max;
			     restore_regs();
			     return 0;
			     }
			  }
		       else {
			 str[c]=chChar ;
			 if ( c == len ) {
			    len++;
			    str[len]=0;
			    }
			 cputch(chChar);
			 c++;
			 if ( c == max ) {
			     SetCursorShape(0);
			     pLength->cchIn = max;
			     restore_regs();
			     return 0 ;
			     }
			 }
	  }
	}
     gotoxy(start_c+c,r);
     }

} /* INPUTSTRING */


/****************************************************************************
 TermVioKbd

     Restore the previously saved cursor data. This routine is a callable
     from outside the DLL.

****************************************************************************/
void far pascal TermVioKbd()
{
  VioSetCurType(&viocurstat,0);
  VioSetState(&viostat,0);
} /* TermVioKbd */


/****************************************************************************
 SetCursorShape

     Sets the cursor shape. This routine is a NOT callable form outside
     the DLL.

 Input :
    shape : 0 - underline , 1 - block , 2 - no cursor

****************************************************************************/
void SetCursorShape ( int shape )
{
  VIOCURSORINFO tviocurstat ;

  memcpy(&tviocurstat,&viocurstat,sizeof(VIOCURSORINFO) );

  tviocurstat.attr= (shape==2)? -1:0;

  if ( shape ) {
     tviocurstat.yStart= 0 ;
     tviocurstat.cEnd= viofinfo.cyCell-1;
     }
  else {
     tviocurstat.yStart= viofinfo.cyCell-3 ;
     tviocurstat.cEnd= viofinfo.cyCell-2;
     }

  VioSetCurType(&tviocurstat,0);

} /* SetCursorShape */


